//==================================================================//
//==================  =  1.0 = (04.01.2011) ==========//
//==================================================================//
//        DS1307:             //
// 1. ds1307_init -    RTC                      //
// 2. ds1307_get_time -                        //
// 3. ds1307_get_date -                  //
// 4. ds1307_set_time -                  //
// 5. ds1307_set_date -                      //
//==================================================================//

extern void ds1307_init (void);
extern void ds1307_get_time (char *hr, char *mn, char *sec);
extern void ds1307_get_date (char *day, char *mth, char *year, char *dow);
extern void ds1307_set_time (char hr, char mn, char sec);
extern void ds1307_set_date (char day, char mth, char year);

#define ACK  1
#define NACK 0
#define DS1307_ADDR 0xD0

void ds1307_init (void) {
 char sec_t = 0;
 char sec = 0;
 
 I2C1_Start();
 I2C1_Wr(DS1307_ADDR);      // WR to RTC
 I2C1_Wr(0x00);             // REG 0
 
 I2C1_Repeated_Start();
 I2C1_Wr(DS1307_ADDR + 1);
 sec_t = I2C1_Rd(NACK);     // REG 0 - reading current seconds
 while (!I2C1_Is_Idle())
  asm nop;
 I2C1_Stop();
 
 sec = Dec2Bcd((Bcd2Dec(sec_t)) & 0x7F);
 Delay_ms(50);
 
 I2C1_Start();
 I2C1_Wr(DS1307_ADDR);      // WR to RTC
 I2C1_Wr(0x00);
 I2C1_Wr(sec);              // Write current seconds to REG 0 and start oscillator
 while (!I2C1_Is_Idle())
  asm nop;
 //I2C1_Wr(0x00);             // WR 00000000 to REG 0 (start RTC with 00 seconds)
 I2C1_Repeated_Start();
 I2C1_Wr(DS1307_ADDR);      // WR to RTC
 I2C1_Wr(0x07);             // Control Register
 I2C1_Wr(0x80);             // Disable squarewave output pin
 while (!I2C1_Is_Idle())
  asm nop;
 I2C1_Stop();
}

void ds1307_get_time (char *hr, char *mn, char *sec) {
 char sec_t = 0;
 char mn_t = 0;
 char hr_t = 0;

 I2C1_Start();
 I2C1_Wr(DS1307_ADDR);          // WR to RTC
 I2C1_Wr(0x00);                 // REG 0
 I2C1_Repeated_Start();
 I2C1_Wr(DS1307_ADDR + 1);
 sec_t = I2C1_Rd(ACK);          // REG 0 - seconds
 while (!I2C1_Is_Idle())
  asm nop;
 mn_t  = I2C1_Rd(ACK);          // REG 1 - minutes
 while (!I2C1_Is_Idle())
  asm nop;
 hr_t  = I2C1_Rd(NACK);         // REG 2 - hours
 while (!I2C1_Is_Idle())
  asm nop;
 I2C1_Stop();
 *sec  = Bcd2Dec(sec_t & 0x7F);
 *mn   = Bcd2Dec(mn_t  & 0x7F);
 *hr   = Bcd2Dec(hr_t  & 0x3F);
}

void ds1307_get_date (char *day, char *mth, char *year, char *dow) {
 char day_t  = 0;
 char mth_t  = 0;
 char year_t = 0;
 char dow_t  = 0;

 I2C1_Start();
 I2C1_Wr(DS1307_ADDR);            // WR to RTC
 I2C1_Wr(0x03);                   // REG 3
 I2C1_Repeated_Start();
 I2C1_Wr(DS1307_ADDR + 1);
 dow_t   = I2C1_Rd(ACK);          // REG 3 - day of week
 while (!I2C1_Is_Idle())
  asm nop;
 day_t   = I2C1_Rd(ACK);          // REG 4 - day
 while (!I2C1_Is_Idle())
  asm nop;
 mth_t   = I2C1_Rd(ACK);          // REG 5 - month
 while (!I2C1_Is_Idle())
  asm nop;
 year_t  = I2C1_Rd(NACK);         // REG 6 - year
 while (!I2C1_Is_Idle())
  asm nop;
 I2C1_Stop();
 *day    = Bcd2Dec(day_t  & 0x3F);
 *mth    = Bcd2Dec(mth_t  & 0x1F);
 *year   = Bcd2Dec(year_t);
 *dow    = Bcd2Dec(dow_t  & 0x7F);
}

void ds1307_set_time (char hr, char mn, char sec) {
 I2C1_Start();
 I2C1_Wr(DS1307_ADDR);            // WR to RTC
 I2C1_Wr(0x00);                   // REG 0
 I2C1_Wr(Dec2Bcd(sec & 0x7F));    // REG 0 - seconds
 while (!I2C1_Is_Idle())
  asm nop;
 I2C1_Wr(Dec2Bcd(mn  & 0x7F));    // REG 1 - minutes
 while (!I2C1_Is_Idle())
  asm nop;
 I2C1_Wr(Dec2Bcd(hr  & 0x3F));    // REG 2 - hours
 while (!I2C1_Is_Idle())
  asm nop;
 I2C1_Stop();
}

void void ds1307_set_date (char day, char mth, char year) {
 I2C1_Start();
 I2C1_Wr(DS1307_ADDR);            // WR to RTC
 I2C1_Wr(0x04);                   // REG 4
 I2C1_Wr(Dec2Bcd(day  & 0x3F));   // REG 4 - day
 while (!I2C1_Is_Idle())
  asm nop;
 I2C1_Wr(Dec2Bcd(mth  & 0x1F));   // REG 5 - month
 while (!I2C1_Is_Idle())
  asm nop;
 I2C1_Wr(Dec2Bcd(year));          // REG 6 - year
 while (!I2C1_Is_Idle())
  asm nop;
 I2C1_Stop();
}